// The SFML graphic module (sf::RenderWindow) is more sophisticated than the SFML OpenGL window (sf::Window).
// - It adds features to draw SFML specific entities, but it requires extra care when mixing with native OpenGL calls.
// - It accepts OpenGL version 1.1 and above, while the SFML window (sf::Window) requires OpenGL version 3.3 at least.
// The SFML OpenGL window (sf::Window) doesn't necessarily require the TGUI_HAS_FONT_BACKEND_FREETYPE to be set.
// It works fine with the TGUI_HAS_FONT_BACKEND_SFML_GRAPHICS, if TGUI_HAS_RENDERER_BACKEND_SFML_GRAPHICS is also set.
// See "https://www.sfml-dev.org/tutorials/2.5/window-opengl.php" (last section) for details.
#include <TGUI/Config.hpp>
#include <TGUI/TGUI.hpp>

#ifdef TGUI_HAS_BACKEND_SFML_GRAPHICS
#include <TGUI/Backend/SFML-Graphics.hpp>
#else
#include <TGUI/Backend/SFML-OpenGL3.hpp>
#endif

#include "../TGUI-0.10-Extensions/ImageBytes.hpp"
#include "../TGUI-0.10-Extensions/RibbonButton.hpp"
#include "../TGUI-0.10-Extensions/PortableFileDialogsWrapper.hpp"

#include "../TGUI-0.10-Extensions/MainFrame.hpp"

#include <iostream>
#include <fstream>
#include <thread>
#include <cmath>

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief My derived application main form
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class Bin2HeaderMainForm : public tgui::MainForm
{
private:
    tgui::TabContainer::Ptr m_menuTabContainer;         //!< The menu tab container.
    tgui::TextArea::Ptr     m_textArea;                 //!< The text area.
    tgui::FileDialog::Ptr   m_fileOpenDialog;           //!< The file open dialog.
    tgui::MessageBox::Ptr   m_infoMessageBox;           //!< The info message box.

    const float             m_tabFixedSize = 150.0f;    //!< The fixed size of the tab.

private:
    Bin2HeaderMainForm() = default;

public:
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /// @brief Constructor
    ///
    /// @param window     The main form window
    /// @param gui        The back-end gui (containing native window and renderer)
    /// @param clearColor The background color of the back-end window
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    Bin2HeaderMainForm(sf::Window& window, tgui::BackendGui& gui, tgui::Color clearColor = tgui::Color(240, 240, 240)) :
        MainForm(window, gui, clearColor)
    {
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /// @brief Close the application
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    void buttonFileLogOutCallBack()
    {
        // This is a very simple approach, that doesn't support the "Do you really want to close?" question.
        getWindow().close();
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /// @brief Convert a binary file to a C++ header
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    tgui::String convertBin2Header(std::ifstream& source, tgui::String name)
    {
        const int chunkSize = 1024 * 1024; // default 1MB

		unsigned long long dataLength;
		source.seekg(0, std::ifstream::end);
		dataLength = source.tellg();
		source.seekg(0, std::ifstream::beg);

		unsigned long long chunkCount = ceil((double) dataLength / chunkSize);

		bool eof = false;
		unsigned long long bytesWritten = 0;
		unsigned long long chunkIdx;
        tgui::String result(U"        new std::uint8_t[] {\n");

		for (chunkIdx = 0; chunkIdx < chunkCount; chunkIdx++)
        {
            if (chunkIdx != chunkCount - 1)
                continue;

			if (eof)
				break;

			char chunk[chunkSize];
			source.seekg(chunkIdx * chunkSize);
			source.read(chunk, chunkSize);

			unsigned int byteIdx;
			for (byteIdx = 0; byteIdx < chunkSize; byteIdx++)
            {
				if ((bytesWritten % 16) == 0)
				{
					result.append(U"            ");
				}

				std::stringstream ssByte;
				ssByte << "0x" << std::hex << std::setw(2) << std::setfill('0') << (int) (unsigned char) chunk[byteIdx];
				result.append(ssByte.str());
				bytesWritten++;

				if (((bytesWritten - 1) % 16) == 15)
					result.append(U",\n");
				else if (bytesWritten >= dataLength)
                {
					eof = true;
					break;
				}
				else
					result.append(U", ");
			}
		}

        result.append(U"\n        };\n");
        std::stringstream ssSize;
        ssSize << dataLength;
        result.append(U"        SIZE = ");
        result.append(ssSize.str());
        result.append(U";");
        return result;
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /// @brief Call the TGUI internal file dialog
    ///
    /// See also: https://forum.tgui.eu/index.php?topic=789.msg3894#msg3894
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    void buttonFileOpenCallBack()
    {
        std::vector<tgui::String> imageFileExtensions;
        if (m_fileOpenDialog == nullptr)
        {
            imageFileExtensions.push_back(U"*.png");
            imageFileExtensions.push_back(U"*.jpg");
            std::vector<std::pair<tgui::String, std::vector<tgui::String>>> filters;
            filters.push_back(std::pair<tgui::String, std::vector<tgui::String>>(U"Images (png, jpg)", imageFileExtensions));
            filters.push_back(std::pair<tgui::String, std::vector<tgui::String>>(U"All files", std::vector<tgui::String>({U"*.*"})));

            m_fileOpenDialog = tgui::FileDialog::create(U"Open file", U"Open");
            m_fileOpenDialog->onClosing(&Bin2HeaderMainForm::onClosingFileOpenCallBack, this);
            m_fileOpenDialog->setFileTypeFilters(filters, 0);
        }

        m_fileOpenDialog->setPosition("5%", "5%");
        m_fileOpenDialog->setSize("90%", "90%");

        this->getGui().add(m_fileOpenDialog, U"FileOpenDialog");
        // Closing the ChildWindow (FileDialog) just means to remove it from GUI, the class instance will not be destroyed.

        return;
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /// @brief Process the selected file
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    bool onClosingFileOpenCallBack()
    {
        if (m_fileOpenDialog != nullptr)
        {
            auto selectedPaths = m_fileOpenDialog->getSelectedPaths();
            if (selectedPaths.size() > 0)
            {
                auto selectedPath = selectedPaths[0];
                std::ifstream selectedFileStream(selectedPath.asString().toStdString());
                if (!selectedFileStream.is_open())
                    return false;
                selectedFileStream.close();

                selectedFileStream.open(selectedPath.asString().toStdString(), std::ifstream::binary);
                auto result = convertBin2Header(selectedFileStream, selectedPath.getFilename());
                selectedFileStream.close();
                m_textArea->setText(result);

                return true;
            }
        }
        return false;
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /// @brief Close the TGUI internal message box
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    bool buttonInfoMessageBoxCallBack()
    {
        return this->getGui().remove(m_infoMessageBox);
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /// @brief Call the TGUI internal message box
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    void buttonInfoCallBack()
    {
        if (m_infoMessageBox == nullptr)
        {
            tgui::String message(U"This is 'Bin2HeaderTGUI', a TGUI sample application version 0.1.\n\n");
            message.append(U"It shows some basic TGUI capabilities like:\n");
            message.append(U"- TabContainer and RibbonButton with icon,\n");
            message.append(U"- internal FileDialog to open a file including filter,\n");
            message.append(U"- internal MessageBox and\n");
            message.append(U"- mono-space Font assignment to the TextArea.\n");
            std::vector<tgui::String> buttons = {U"OK"};
            m_infoMessageBox = tgui::MessageBox::create(U"Bin2HeaderTGUI 0.1", message, buttons);
            m_infoMessageBox->onButtonPress(&Bin2HeaderMainForm::buttonInfoMessageBoxCallBack, this);
        }

        m_infoMessageBox->setPosition("25%", "25%");
        m_infoMessageBox->setSize("50%", "50%");

        this->getGui().add(m_infoMessageBox, U"InfoMessageBox");
        // Closing the ChildWindow (MessageBox) just means to remove it from GUI, the class instance will not be destroyed.

        return;
    }

    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /// @brief Create the widgets of the main frame
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    bool createFrameContent()
    {
        const float groupWidth = 100.0f;

        // Section 1: Prepare TabContainer to be used as ribbon.
        m_menuTabContainer = tgui::TabContainer::create();
        m_menuTabContainer->setPosition(0.0f, 0.0f);
        m_menuTabContainer->setSize("100%", "84");
        m_menuTabContainer->setTabFixedSize(m_tabFixedSize);
        m_menuTabContainer->setTabAlignment(tgui::TabContainer::TabAlign::Bottom);
        auto tabPanel1 = m_menuTabContainer->addTab(U"Start", true);
        auto tabPanel2 = m_menuTabContainer->addTab(U"Edit", false);
        this->getGui().add(m_menuTabContainer, U"MenuTabContainer");

        // Section 2: Fill first ribbon tab.
        auto systemLogOutButton = tgui::RibbonButton::create(tangoIconTheme::Icons28::Action_SystemLogOut.getBytes(), tangoIconTheme::Icons28::Action_SystemLogOut.getByteCount());
        systemLogOutButton->setPosition(m_ribbonWidgetSpacing.x, m_ribbonWidgetSpacing.y);
        systemLogOutButton->setSize(groupWidth - m_ribbonWidgetSpacing.x * 2, 28.0f);
        systemLogOutButton->setText(U"LogOut");
        tabPanel1->add(systemLogOutButton, U"RibbonFile_LogOutButton");
        systemLogOutButton->onMouseRelease(&Bin2HeaderMainForm::buttonFileLogOutCallBack, this);

        // Section 2a: Separate ribbon groups.
        auto separator1 = tgui::SeparatorLine::create();
        separator1->setPosition(groupWidth, 0.0f);
        separator1->setSize(2.0f, "100%");
        tabPanel1->add(separator1, U"RibbonFile_Separator1");
        auto separator1Renderer = separator1->getRenderer();
        separator1Renderer->setColor(tgui::Color(192, 192, 192, 255));

        auto fileOpenButton = tgui::RibbonButton::create(tangoIconTheme::Icons28::Action_DocumentOpen.getBytes(), tangoIconTheme::Icons28::Action_DocumentOpen.getByteCount());
        fileOpenButton->setPosition(groupWidth + m_ribbonWidgetSpacing.x, m_ribbonWidgetSpacing.y);
        fileOpenButton->setSize(groupWidth - m_ribbonWidgetSpacing.x * 2, 28.0f);
        fileOpenButton->setText(U"Open");
        tabPanel1->add(fileOpenButton, U"RibbonFile_OpenButton");
        fileOpenButton->onMouseRelease(&Bin2HeaderMainForm::buttonFileOpenCallBack, this);

        // Section 2b: Separate ribbon groups.
        auto separator2 = tgui::SeparatorLine::create();
        separator2->setPosition(2 * groupWidth, 0.0f);
        separator2->setSize(2.0f, "100%");
        tabPanel1->add(separator2, U"RibbonFile_Separator2");
        auto separator2Renderer = separator2->getRenderer();
        separator2Renderer->setColor(tgui::Color(192, 192, 192, 255));

        auto infoBrowserButton = tgui::RibbonButton::create(tangoIconTheme::Icons28::Apps_InfoBrowser.getBytes(), tangoIconTheme::Icons28::Apps_InfoBrowser.getByteCount());
        infoBrowserButton->setPosition(groupWidth * 2 + m_ribbonWidgetSpacing.x, m_ribbonWidgetSpacing.y);
        infoBrowserButton->setSize(groupWidth -  + m_ribbonWidgetSpacing.x * 2, 28.0f);
        infoBrowserButton->setText(U"Info");
        tabPanel1->add(infoBrowserButton, U"RibbonFile_InfoButton");
        infoBrowserButton->onMouseRelease(&Bin2HeaderMainForm::buttonInfoCallBack, this);

        // Section 3: Fill second ribbon tab.
        auto editCopyButton = tgui::RibbonButton::create(tangoIconTheme::Icons28::Action_EditCopy.getBytes(), tangoIconTheme::Icons28::Action_EditCopy.getByteCount());
        editCopyButton->setPosition(m_ribbonWidgetSpacing.x, m_ribbonWidgetSpacing.y);
        editCopyButton->setSize(groupWidth, 28.0f);
        editCopyButton->setText(U"Copy");
        tabPanel2->add(editCopyButton, U"RibbonCopyButton");

        auto editPasteButton = tgui::RibbonButton::create(tangoIconTheme::Icons28::Action_EditPaste.getBytes(), tangoIconTheme::Icons28::Action_EditPaste.getByteCount());
        editPasteButton->setPosition(m_ribbonWidgetSpacing.x, 32.0f);
        editPasteButton->setSize(groupWidth, 28.0f);
        editPasteButton->setText(U"Paste");
        tabPanel2->add(editPasteButton, U"RibbonPasteButton");

        // Section 4: Fill work area.
        m_textArea = tgui::TextArea::create();
        m_textArea->setPosition("3%", "100");
        tgui::Layout hightLayout(tgui::Layout::Operation::Minus, std::make_unique<tgui::Layout>("100%"), std::make_unique<tgui::Layout>("140"));
        m_textArea->setSize("94%", hightLayout);
    std::string fontPath = "/usr/share/fonts/truetype/Hack-Regular.ttf";
    std::ifstream themeFile(fontPath);
    if (!themeFile.is_open())
        std::cerr << "ERROR: Unable to load mono-space font.\n";
    else
    {
        auto textAreaRenderer = m_textArea->getRenderer();
#ifdef TGUI_HAS_BACKEND_SFML_GRAPHICS
        std::shared_ptr<tgui::BackendFont> monospaceBackendFont(new tgui::BackendFontSFML());
#else
        std::shared_ptr<tgui::BackendFont> monospaceBackendFont(new tgui::BackendFontFreetype());
#endif
        monospaceBackendFont->loadFromFile(fontPath);
        tgui::Font monospaceFont(monospaceBackendFont, U"Hack-Regular");
        textAreaRenderer->setFont(monospaceFont);
    }

        m_textArea->setText(U"Hello reader,\n\nthis is a multiline text editor.");
        this->getGui().add(m_textArea, "TextArea");

        // Section 5: Create status bar.
        this->setStatusText(L"Hello, I am a state text!");
        return true;
    }
};

// Run main(int, char**) instead main() to be able to deug in Code::Blocks.
int main(int argc, char** argv)
{
    // Section 1: Native window creation.
#ifdef TGUI_HAS_BACKEND_SFML_GRAPHICS
    sf::RenderWindow window(sf::VideoMode(980, 600), "TGUI window", sf::Style::Default);
#else
    // The OpenGL renderer backend in TGUI requires at least OpenGL version 3.3.
    // Request the OpenGL version 3.3, even if it isn't provided by the hardware.
    // Sometimes you'll have luck. (Mesa 20.2.5 pretends 3.3 on my 3.0 hardware.)
    sf::ContextSettings requestedettings;
    requestedettings.attributeFlags = sf::ContextSettings::Attribute::Core;
    requestedettings.majorVersion = 3;
    requestedettings.minorVersion = 3;

    sf::Window window(sf::VideoMode(980, 600), "TGUI window", sf::Style::Default, requestedettings);
#endif
    auto realizedSettings = window.getSettings();
    std::cout << "SUCCESS creating main frame window with OpenGL " << realizedSettings.majorVersion << "." << realizedSettings.minorVersion << " context.\n";

    // Section 2: TGUI initialization.
    tgui::Gui gui(window);
    std::string themeFilePath = "./themes/BabyBlue.txt";
    std::ifstream themeFile(themeFilePath);
    if (!themeFile.is_open())
        std::cerr << "ERROR: Unable to load theme.\n";
    else
        tgui::Theme::setDefault(themeFilePath);
    auto container = gui.getContainer();

    // Section 2: Sample application.
    Bin2HeaderMainForm mainForm(window, gui);
    if (!mainForm.createFrameContent())
    {
        std::cerr << "ERROR: Unable to create window content.\n";
        return EXIT_FAILURE;
    }
    std::cout << "SUCCESS creating window content.\n";

    // Section 4: Generic application infrastructure.
    gui.mainLoop(mainForm.getClearColor());

    return EXIT_SUCCESS;
}
