/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// TGUIEX - Texus' Graphical User Interface Extensions
// Copyright (C) 2022 Steffen Ploetz (steffen.ploetz@cityweb.com)
//
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Required includes: TGUI/Config.hpp
//                    TGUI/Backend/SFML-Graphics.hpp
//                    TGUI/Timer.hpp
//                    thread

#ifndef TGUIEX_GUIEX_HPP
#define TGUIEX_GUIEX_HPP

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace tgui
{
    inline namespace SFML_GRAPHICS
    {
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Extended Gui class, derived from the librarie's standard gui class for SFML GRAPHICS backend (which is using
        /// the sfml-graphics module for rendering and font). Can neither serve for SFML OPENGL3 nor for GLFW or SDL backend!
        ///
        /// This class is derived to override tgui::SFML_GRAPHICS::Gui::mainLoop().
        /// (The underlaying tgui::SFML_GRAPHICS::Gui is derived from tgui::SFML_GRAPHICS::BackendGuiSFML.)
        /// The tgui::SFML_GRAPHICS::BackendGuiSFML::mainLoop() should regularly be checked for updates.
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        class TGUI_API GuiEx : public tgui::SFML_GRAPHICS::Gui
        {
        public:

            /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            /// @brief Default constructor
            ///
            /// @warning You must still call setWindow (or setTarget) on the Gui before using any TGUI functions
            /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            GuiEx() = default;


            /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            /// @brief Constructor that immediately sets the window on which the gui should be drawn
            ///
            /// @param window  The SFML window that will be used by the gui
            /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            GuiEx(sf::RenderWindow& window)
                : Gui(window)
            {
                ;
            }


            /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            /// @brief Give the gui control over the main loop
            ///
            /// @param clearColor  background color of the window
            ///
            /// This function is only intended in cases where your program only needs to respond to gui events.
            /// For multimedia applications, games, or other programs where you want a high framerate or do a lot of processing
            /// in the main loop, you should use your own main loop.
            /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            void mainLoop(Color clearColor = {240, 240, 240}) override;


            void setToBeClosed(bool toBeClosed)
            {
                m_toBeClosed = toBeClosed;
            }

            /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            /// @brief Get the main form window
            ///
            /// @return The main form window
            /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            sf::Window* getWindow()
            {
                return m_window;
            }

        private:
            bool m_toBeClosed = false; //!< Enable a controlled exit.
        };
    }
} // namespace tgui

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif // TGUIEX_GUIEX_HPP
