/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// TGUIEX - Texus' Graphical User Interface Extensions
// Copyright (C) 2022 Steffen Ploetz (steffen.ploetz@cityweb.com)
//
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Required includes: TGUI/TGUI.hpp

#ifndef TGUIEX_MAINFRAME_HPP
#define TGUIEX_MAINFRAME_HPP

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace tgui
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /// @brief A very simple base class for an application main form
    ///
    /// You can derive your specific main form from this base class or create your own
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    class MainForm
    {
    public:
        typedef std::shared_ptr<MainForm> Ptr;            //!< Shared main form pointer
        typedef std::shared_ptr<const MainForm> ConstPtr; //!< Shared constant main form pointer

    private:
        sf::Window&       m_window;                       //!< The main frame window
        tgui::BackendGui& m_gui;                          //!< The underlying GUI, that organizes the widgets
        tgui::Color       m_clearColor;                   //!< The main form background color
        tgui::Panel::Ptr  m_stausPanel;                   //!< The status panel
        tgui::Label::Ptr  m_statusLabel;                  //!< The main label within the status panel

    protected:
        tgui::Vector2f    m_ribbonWidgetSpacing;          //!< The horizontal spacing between ribbon widgets

    private:
        MainForm() = default;                             //!< The hidden default constructor

    public:
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Constructor
        ///
        /// Prefer the constructor in cases where the main frame needs not to be shared, otherwise prefer the factory method.
        ///
        /// @param window     The main form window
        /// @param gui        The backend gui (containing native window and renderer)
        /// @param clearColor The background color of the backend window
        ///
        /// @see create
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        MainForm(sf::Window& window, tgui::BackendGui& gui, tgui::Color clearColor = tgui::Color(240, 240, 240)) :
           m_window(window),  m_gui(gui), m_clearColor(clearColor), m_ribbonWidgetSpacing(4.0f, 4.0f)
        {
            m_statusLabel = tgui::Label::create();
            auto labelHeight = (int)(m_statusLabel->getTextSize() * 1.4f);

            m_stausPanel = tgui::Panel::create();
            m_stausPanel->setPosition("0", "parent.innerheight - height");
            m_stausPanel->setSize("100%", labelHeight + 2);
            getGui().add(m_stausPanel, "MainForm-StatusPanel");

            auto statusPanelRenderer = m_stausPanel->getRenderer();
            statusPanelRenderer->setBorders(tgui::Outline(1, 1));
            statusPanelRenderer->setBorderColor(tgui::Color(192, 192, 192));
            statusPanelRenderer->setBackgroundColor(getClearColor());

            m_statusLabel->setSize("100%", "100%");
            m_stausPanel->add(m_statusLabel, "MainForm-StatusLabel");
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Creates a new main form
        ///
        /// Prefer the factory method in cases where the main frame needs to be shared, otherwise prefer the constructor.
        ///
        /// @param window     The main form window
        /// @param gui        The back-end GUI (containing native window and renderer)
        /// @param clearColor The background color of the back-end window
        ///
        /// @return The new main form
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        static MainForm::Ptr create(sf::Window& window, tgui::BackendGui& gui, tgui::Color clearColor = tgui::Color(240, 240, 240))
        {
            auto mainForm = std::make_shared<MainForm>(window, gui, clearColor);
            return mainForm;
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Returns the back-end GUI (containing native window and renderer)
        ///
        /// @return Back-end GUI (containing native window and renderer)
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        tgui::BackendGui& getGui() const
        {   return m_gui;    }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Sets the background color of the back-end window
        ///
        /// @param clearColor Background color of the back-end window
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        void setClearColor(tgui::Color clearColor)
        {   m_clearColor = clearColor;   }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Returns the background color of the back-end window
        ///
        /// @return Background color of the back-end window
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        tgui::Color getClearColor() const
        {   return m_clearColor;   }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Get the main form window
        ///
        /// @return The main form window
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        sf::Window& getWindow()
        {   return m_window;    }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Sets the label text of the status bar
        ///
        /// @param text Label text of the status bar
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        void setStatusText(tgui::String text)
        {   m_statusLabel->setText(text);    }
    };
} // namespace tgui

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif // TGUIEX_MAINFRAME_HPP

