/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// TGUIEX - Texus' Graphical User Interface Extensions
// Copyright (C) 2022 Steffen Ploetz (steffen.ploetz@cityweb.com)
//
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// See: https://github.com/samhocevar/portable-file-dialogs

// Required includes: TGUI/TGUI.hpp

#ifndef TGUIEX_PORTABLE_FILE_DIALOG_HPP
#define TGUIEX_PORTABLE_FILE_DIALOG_HPP

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace tgui
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    /// @brief A very simple base class for an application main form
    ///
    /// You can derive your specific main form from this base class or create your own
    /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    class PortableFileDialog
    {
    public:
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		/// @brief Enumeration of buttons, that can be returned as result from a dialog
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		enum Buttons
		{
			Cancel           = -1,  //!< Dialog cosed with Cancel button
			Ok               =  0,  //!< Dialog cosed with OK button
			Yes              =  1,  //!< Dialog cosed with Yes button
			No               =  2,  //!< Dialog cosed with No button
			Abort            =  3,  //!< Dialog cosed with Abort button
			Retry            =  4,  //!< Dialog cosed with Retry button
			Ignore           =  5   //!< Dialog cosed with Ignore button
		};


        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		/// @brief Enumeration of buttons, that can be displayed by a dialog
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		enum Choice
        {
			OK                 = 0,   //!< Show the OK button
            OK_Cancel          = 1,   //!< Show the OK and Cancel buttons
            Yes_No             = 2,   //!< Show the Yes and No buttons
            Yes_No_Cancel      = 3,   //!< Show the Yes, No and Cancel buttons
            Retry_Cancel       = 4,   //!< Show the Retry and Cancel buttons
            Abort_Retry_Ignore = 5    //!< Show the Abort, Retry and Cancel buttons
        };


        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		/// @brief Enumeration of icons, that can be displayed by a dialog
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		enum Icon
        {
            Info                = 0,   //!< Show the Info icon
            Warning             = 1,   //!< Show the Warning icon
            Error               = 2,   //!< Show the Error icon
            Question            = 3    //!< Show the Question icon
        };



        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		/// @brief Additional option flags for various dialog constructors
		/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        enum FileDialogOptions : uint8_t
        {
            None            = 0,
            Multiselect     = 0x1, //!< For file open, allow multiselect.
            ForceOverwrite  = 0x2, //!< For file save, force overwrite and disable the confirmation dialog.
            ForcePath       = 0x4, //!< For folder select, force path to be the provided argument instead of the last opened directory
            // Which is the Microsoft-recommended, user-friendly behaviour.
        };

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief First of four default file type filters
        ///
        /// @return Returns the file type filter for all files
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        static std::vector<std::string> allFilesFilter()
        {   return { "All Files", "*" };    }


        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Second of four default file type filters
        ///
        /// @return Returns the file type filter for image files
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        static std::vector<std::string> imageFilesFilter()
        {   return { "Image Files", "*.png *.jpg *.jpeg *.bmp" };    }


        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Third of four default file type filters
        ///
        /// @return Returns the file type filter for audio files
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        static std::vector<std::string> audioFilesFilter()
        {   return { "Audio Files", "*.wav *.mp3" };    }


        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Fourth of four default file type filters
        ///
        /// @return Returns the file type filter for media files
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        static std::vector<std::string> mediaFilesFilter()
        {   return { "Image Files", "*.png *.jpg *.jpeg *.bmp", "Audio Files", "*.wav *.mp3" };    }


        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Check whether portable dialogs are supported
        ///
        /// @return Returns @code true @endcode if portable dialogs are supported or @code false @endcode otherwise
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        static bool requirementsMet();


        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Open a message box
        ///
        /// @param title       The dialog title
        /// @param message     The message to display
        /// @param choice      The combination of buttons to display
        /// @param icon        The icon to display
        ///
        /// @return Returns the code of the button, that closes the message box
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        static PortableFileDialog::Buttons message(std::string const &title = "Hello",
                                                   std::string const &message = "Do you want to proceed?",
                                                   PortableFileDialog::Choice choice = PortableFileDialog::Choice::OK,
                                                   PortableFileDialog::Icon icon = PortableFileDialog::Icon::Info);


        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Open a popup notification
        ///
        /// @param title       The notification title
        /// @param text        The text to display
        /// @param icon        The icon to display
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        static void notify(std::string const &title = "Hello",
                           std::string const &text = "Something might be on fire 🔥",
                           PortableFileDialog::Icon icon = PortableFileDialog::Icon::Warning);


        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Open the file open dialog
        ///
        /// @param title       The dialog title
        /// @param initialPath The path to show when dialog opens
        /// @param filter      The The file type filter
        /// @param options     The selection options, valid are @code None @endcode and @code Multiselect @endcode
        ///
        /// @return The collection of selected files on success or an empty collection otherwise
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        static std::vector<std::string> fileOpen(std::string const &title = "Select file to open",
                                                 std::string const &initialPath = ".",
                                                 std::vector<std::string> filter = mediaFilesFilter(),
                                                 PortableFileDialog::FileDialogOptions options = PortableFileDialog::FileDialogOptions::None);


        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Open the file save dialog
        ///
        /// @param title       The dialog title
        /// @param initialPath The path to show when dialog opens
        /// @param filter      The The file type filter
        /// @param options     The selection options, valid are @code None @endcode and @code ForceOverwrite @endcode
        ///
        /// @return The destination (path and file name) on success or an empty string ( @code empty() @endcode ) otherwise
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        static std::string fileSave(std::string const &title = "Select destination to save to",
                                    std::string const &initialPath = ".",
                                    std::vector<std::string> filter = mediaFilesFilter(),
                                    PortableFileDialog::FileDialogOptions options = PortableFileDialog::FileDialogOptions::None);


        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Open the select folder dialog
        ///
        /// @param title       The dialog title
        /// @param initialPath The path to show when dialog opens
        /// @param options     The selection options, valid are @code None @endcode and @code ForcePath @endcode
        ///
        /// @return The destination (path and file name) on success or an empty string ( @code empty() @endcode ) otherwise
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        static std::string folderSelect(std::string const &title = "Select folder",
                                        std::string const &initialPath = ".",
                                        PortableFileDialog::FileDialogOptions options = PortableFileDialog::FileDialogOptions::ForcePath);
    };

} // namespace tgui

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif // TGUIEX_PORTABLE_FILE_DIALOG_HPP
