/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// TGUIEX - Texus' Graphical User Interface Extensions
// Copyright (C) 2022 Steffen Ploetz (steffen.ploetz@cityweb.com)
//
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Required includes: TGUI/TGUI.hpp

#ifndef TGUIEX_RIBBONBUTTON_HPP
#define TGUIEX_RIBBONBUTTON_HPP

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace tgui
{

    #define TRANSPARENT_HEIGHT 28
    #define TRANSPARENT_WIDTH 28
    #define TRANSPARENT_SIZE 104
    static const std::uint8_t transparent_png[] = {
        0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a, 0x00, 0x00, 0x00, 0x0d,
        0x49, 0x48, 0x44, 0x52, 0x00, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x00, 0x1c,
        0x08, 0x06, 0x00, 0x00, 0x00, 0x72, 0x0d, 0xdf, 0x94, 0x00, 0x00, 0x00,
        0x09, 0x70, 0x48, 0x59, 0x73, 0x00, 0x00, 0x0b, 0x13, 0x00, 0x00, 0x0b,
        0x13, 0x01, 0x00, 0x9a, 0x9c, 0x18, 0x00, 0x00, 0x00, 0x1a, 0x49, 0x44,
        0x41, 0x54, 0x48, 0x89, 0xed, 0xc1, 0x31, 0x01, 0x00, 0x00, 0x00, 0xc2,
        0xa0, 0xf5, 0x4f, 0x6d, 0x08, 0x5f, 0xa0, 0x00, 0x00, 0x00, 0x7e, 0x03,
        0x0c, 0x5c, 0x00, 0x01, 0xdd, 0x54, 0xd6, 0xba, 0x00, 0x00, 0x00, 0x00,
        0x49, 0x45, 0x4e, 0x44, 0xae, 0x42, 0x60, 0x82
    };

    class TransparentTexture
    {
    private:
        typedef std::shared_ptr<tgui::Texture> Ptr;

        static Ptr m_image;
    public:
        static const tgui::Texture& get()
        {
            if (m_image == nullptr)
            {
                tgui::Texture t;
                t.loadFromMemory(transparent_png, TRANSPARENT_SIZE);
                m_image = std::make_shared<tgui::Texture>(t);
            }
            return *m_image;
        }
    };

    class RibbonButton : public BitmapButton
    {
    public:

        typedef std::shared_ptr<RibbonButton> Ptr; //!< Shared bitmap button extended pointer
        typedef std::shared_ptr<const RibbonButton> ConstPtr; //!< Shared constant bitmap button extended pointer

    private:
        tgui::Texture           m_texture;

    private:
        RibbonButton() = default;

    public:
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @internal
        /// @brief Constructor
        /// @param texturerPath     Path to the texture file. Recommended is a 28px * 28px PNG file with transparency
        /// @see create
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        RibbonButton(String texturePath) :
            BitmapButton()
        {
            m_texture.load(texturePath);
            setImage(m_texture);
            setTextPosition({"0%", "50%"}, {0, 0.5f});
            init();
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @internal
        /// @brief Constructor
        /// @param textureBytes      Bytes of the texture. Recommended is a 28px * 28px PNG file with transparency
        /// @param textureSize       Number of bytes of the texture
        /// @see create
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        RibbonButton(const std::uint8_t* textureBytes, std::size_t textureSize) :
            BitmapButton()
        {
            m_texture.loadFromMemory(textureBytes, textureSize);
            setImage(m_texture);
            setTextPosition({"0%", "50%"}, {0, 0.5f});
            init();
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Creates a new bitmap button extended
        ///
        /// Prefer the factory method in cases where the ribbon button needs to be shared, otherwise prefer the constructor.
        ///
        /// @param texturerPath     Path to the texture file. Recommended is a 28px * 28px PNG file with transparency
        ///
        /// @return The new bitmap button extended
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        static RibbonButton::Ptr create(String texturePath)
        {
            auto ribbonButton = std::make_shared<RibbonButton>(texturePath);
            return ribbonButton;
        }

        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        /// @brief Creates a new bitmap button extended
        ///
        /// Prefer the factory method in cases where the ribbon button needs to be shared, otherwise prefer the constructor.
        ///
        /// @param textureBytes      Bytes of the texture. Recommended is a 28px * 28px PNG file with transparency
        /// @param textureSize       Number of bytes of the normal texture
        ///
        /// @return The new bitmap button extended
        /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        static RibbonButton::Ptr create(const std::uint8_t* textureBytes, std::size_t textureSize)
        {
            auto ribbonButton = std::make_shared<RibbonButton>(textureBytes, textureSize);
            return ribbonButton;
        }

    private:
        void init()
        {
            auto renderer = getRenderer();
            renderer->setTexture(TransparentTexture::get());
            renderer->setTextureDown(TransparentTexture::get());
            renderer->setTextureHover(TransparentTexture::get());
            renderer->setTextureDownHover(TransparentTexture::get());
            renderer->setTextureDisabled(TransparentTexture::get());
            renderer->setTextureDownDisabled(TransparentTexture::get());
            renderer->setTextureFocused(TransparentTexture::get());
            renderer->setTextureDownFocused(TransparentTexture::get());
            renderer->setTextColorHover(tgui::Color::Black);
            renderer->setBorders(tgui::Outline(1));
            renderer->setBorderColor(tgui::Color::Transparent);
            renderer->setBorderColorDown(tgui::Color::Transparent);
            renderer->setBorderColorHover(tgui::Color(128, 128, 128, 255));
            renderer->setBorderColorDownHover(tgui::Color(128, 128, 128, 255));
            renderer->setBorderColorDisabled(tgui::Color::Transparent);
            renderer->setBorderColorDownDisabled(tgui::Color::Transparent);
            renderer->setBorderColorFocused(tgui::Color::Transparent);
            renderer->setBorderColorDownFocused(tgui::Color::Transparent);
        }
    };
} // namespace tgui

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif // TGUIEX_RIBBONBUTTON_HPP
